/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2014 Torus Knot Software Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
-----------------------------------------------------------------------------
*/
#ifndef __GLES2ManagedResource_H__
#define __GLES2ManagedResource_H__

#include "OgreGLES2Prerequisites.h"

namespace Ogre {

#define HANDLE_CONTEXT_LOSS (OGRE_PLATFORM == OGRE_PLATFORM_ANDROID || OGRE_PLATFORM == OGRE_PLATFORM_EMSCRIPTEN)

#if HANDLE_CONTEXT_LOSS
#   define MANAGED_RESOURCE , public GLES2ManagedResource
#   define MANAGED_RESOURCE_SINGLE : public GLES2ManagedResource
    
    class EGLContext;
    
    /** Represents a GLES2 rendering resource.
    Provide unified interface to
    handle various device states.
    */
    class _OgrePrivate GLES2ManagedResource
    {

    // Interface.
    public:

        // Called immediately after the Android context has entered a lost state.
        virtual void notifyOnContextLost() {}

        // Called immediately after the Android context has been reset.
        virtual void notifyOnContextReset() {}

    public:
        GLES2ManagedResource();
        virtual ~GLES2ManagedResource();
    };

#else
#   define MANAGED_RESOURCE
#   define MANAGED_RESOURCE_SINGLE
#endif

}

#endif
